@@ -13,8 +13,9 @@
 package No::Worries::Date;
 use strict;
 use warnings;
-our $VERSION  = "0.2_2";
-our $REVISION = sprintf("%d.%02d", q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/);
+use 5.005; # need the four-argument form of substr()
+our $VERSION  = "0.8_2";
+our $REVISION = sprintf("%d.%02d", q$Revision: 1.12 $ =~ /(\d+)\.(\d+)/);
 
 #
 # used modules
@@ -22,6 +23,8 @@ our $REVISION = sprintf("%d.%02d", q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/);
 
 use HTTP::Date qw(str2time);
 use No::Worries::Die qw(dief);
+use No::Worries::Export qw(export_control);
+use Params::Validate qw(validate_pos :types);
 use POSIX qw(strftime);
 
 #
@@ -39,6 +42,7 @@ sub date_parse ($) {
     my($string) = @_;
     my($time);
 
+    validate_pos(@_, { type => SCALAR });
     $time = str2time($string);
     dief("invalid date: %s", $string) unless defined($time);
     return($time);
@@ -52,14 +56,15 @@ sub date_stamp (;$) {
     my($time) = @_;
     my($string);
 
+    validate_pos(@_, { type => SCALAR }) if @_;
     $time = time() unless defined($time);
     if ($time =~ /^(\d+)$/) {
-	$string = strftime(STRFTIME_STAMP_FORMAT, localtime($1));
+        $string = strftime(STRFTIME_STAMP_FORMAT, localtime($1));
     } elsif ($time =~ /^(\d+)\.(\d+)$/) {
-	$string = strftime(STRFTIME_STAMP_FORMAT, localtime($1));
-	$string .= ".$2";
+        $string = strftime(STRFTIME_STAMP_FORMAT, localtime($1));
+        $string .= ".$2";
     } else {
-	dief("invalid time: %s", $time);
+        dief("invalid time: %s", $time);
     }
     return($string);
 }
@@ -72,14 +77,15 @@ sub date_string (;$) {
     my($time) = @_;
     my($string);
 
+    validate_pos(@_, { type => SCALAR }) if @_;
     $time = time() unless defined($time);
     if ($time =~ /^(\d+)$/) {
-	$string = strftime(STRFTIME_STRING_FORMAT, gmtime($1));
+        $string = strftime(STRFTIME_STRING_FORMAT, gmtime($1));
     } elsif ($time =~ /^(\d+)\.(\d+)$/) {
-	$string = strftime(STRFTIME_STRING_FORMAT, gmtime($1));
-	substr($string, -1, 0) = ".$2";
+        $string = strftime(STRFTIME_STRING_FORMAT, gmtime($1));
+        substr($string, -1, 0, ".$2");
     } else {
-	dief("invalid time: %s", $time);
+        dief("invalid time: %s", $time);
     }
     return($string);
 }
@@ -93,7 +99,7 @@ sub import : method {
 
     $pkg = shift(@_);
     grep($exported{$_}++, map("date_$_", qw(parse stamp string)));
-    No::Worries::_import(scalar(caller()), $pkg, \%exported, @_);
+    export_control(scalar(caller()), $pkg, \%exported, @_);
 }
 
 1;
@@ -164,4 +170,4 @@ L<No::Worries>.
 
 Lionel Cons L<http://cern.ch/lionel.cons>
 
-Copyright CERN 2012
+Copyright (C) CERN 2012-2013
